@@ -511,7 +511,7 @@ fn test_parsing_after_detecting_error_in_the_middle_of_a_string_token() {
     // Delete a suffix of the source code, starting in the middle of the string
     // literal, after some whitespace. With this deletion, the remaining string
     // content: "c, " looks like two valid python tokens: an identifier and a comma.
-    // When this edit is undone, in order correctly recover the orginal tree, the
+    // When this edit is undone, in order correctly recover the original tree, the
     // parser needs to remember that before matching the `c` as an identifier, it
     // lookahead ahead several bytes, trying to find the closing quotation mark in
     // order to match the "string content" node.
@@ -829,6 +829,7 @@ fn test_parsing_with_one_included_range() {
         js_tree.root_node().start_position(),
         Point::new(0, source_code.find("console").unwrap())
     );
+    assert_eq!(js_tree.included_ranges(), &[script_content_node.range()]);
 }
 
 #[test]
@@ -853,28 +854,27 @@ fn test_parsing_with_multiple_included_ranges() {
     let close_quote_node = template_string_node.child(3).unwrap();
 
     parser.set_language(get_language("html")).unwrap();
-    parser
-        .set_included_ranges(&[
-            Range {
-                start_byte: open_quote_node.end_byte(),
-                start_point: open_quote_node.end_position(),
-                end_byte: interpolation_node1.start_byte(),
-                end_point: interpolation_node1.start_position(),
-            },
-            Range {
-                start_byte: interpolation_node1.end_byte(),
-                start_point: interpolation_node1.end_position(),
-                end_byte: interpolation_node2.start_byte(),
-                end_point: interpolation_node2.start_position(),
-            },
-            Range {
-                start_byte: interpolation_node2.end_byte(),
-                start_point: interpolation_node2.end_position(),
-                end_byte: close_quote_node.start_byte(),
-                end_point: close_quote_node.start_position(),
-            },
-        ])
-        .unwrap();
+    let html_ranges = &[
+        Range {
+            start_byte: open_quote_node.end_byte(),
+            start_point: open_quote_node.end_position(),
+            end_byte: interpolation_node1.start_byte(),
+            end_point: interpolation_node1.start_position(),
+        },
+        Range {
+            start_byte: interpolation_node1.end_byte(),
+            start_point: interpolation_node1.end_position(),
+            end_byte: interpolation_node2.start_byte(),
+            end_point: interpolation_node2.start_position(),
+        },
+        Range {
+            start_byte: interpolation_node2.end_byte(),
+            start_point: interpolation_node2.end_position(),
+            end_byte: close_quote_node.start_byte(),
+            end_point: close_quote_node.start_position(),
+        },
+    ];
+    parser.set_included_ranges(html_ranges).unwrap();
     let html_tree = parser.parse(source_code, None).unwrap();
 
     assert_eq!(
@@ -888,6 +888,7 @@ fn test_parsing_with_multiple_included_ranges() {
             " (end_tag (tag_name))))",
         )
     );
+    assert_eq!(html_tree.included_ranges(), html_ranges);
 
     let div_element_node = html_tree.root_node().child(0).unwrap();
     let hello_text_node = div_element_node.child(1).unwrap();
@@ -950,7 +951,9 @@ fn test_parsing_with_included_range_containing_mismatched_positions() {
 
     parser.set_included_ranges(&[range_to_parse]).unwrap();
 
-    let html_tree = parser.parse(source_code, None).unwrap();
+    let html_tree = parser
+        .parse_with(&mut chunked_input(source_code, 3), None)
+        .unwrap();
 
     assert_eq!(html_tree.root_node().range(), range_to_parse);
 
@@ -1077,7 +1080,9 @@ fn test_parsing_with_a_newly_excluded_range() {
     // Parse HTML including the template directive, which will cause an error
     let mut parser = Parser::new();
     parser.set_language(get_language("html")).unwrap();
-    let mut first_tree = parser.parse(&source_code, None).unwrap();
+    let mut first_tree = parser
+        .parse_with(&mut chunked_input(&source_code, 3), None)
+        .unwrap();
 
     // Insert code at the beginning of the document.
     let prefix = "a very very long line of plain text. ";
@@ -1112,7 +1117,9 @@ fn test_parsing_with_a_newly_excluded_range() {
             },
         ])
         .unwrap();
-    let tree = parser.parse(&source_code, Some(&first_tree)).unwrap();
+    let tree = parser
+        .parse_with(&mut chunked_input(&source_code, 3), Some(&first_tree))
+        .unwrap();
 
     assert_eq!(
         tree.root_node().to_sexp(),
@@ -1163,7 +1170,9 @@ fn test_parsing_with_a_newly_included_range() {
     parser
         .set_included_ranges(&[simple_range(range1_start, range1_end)])
         .unwrap();
-    let tree = parser.parse(source_code, None).unwrap();
+    let tree = parser
+        .parse_with(&mut chunked_input(&source_code, 3), None)
+        .unwrap();
     assert_eq!(
         tree.root_node().to_sexp(),
         concat!(
@@ -1180,7 +1189,9 @@ fn test_parsing_with_a_newly_included_range() {
             simple_range(range3_start, range3_end),
         ])
         .unwrap();
-    let tree2 = parser.parse(&source_code, Some(&tree)).unwrap();
+    let tree2 = parser
+        .parse_with(&mut chunked_input(&source_code, 3), Some(&tree))
+        .unwrap();
     assert_eq!(
         tree2.root_node().to_sexp(),
         concat!(
@@ -1288,3 +1299,7 @@ fn simple_range(start: usize, end: usize) -> Range {
         end_point: Point::new(0, end),
     }
 }
+
+fn chunked_input<'a>(text: &'a str, size: usize) -> impl FnMut(usize, Point) -> &'a [u8] {
+    move |offset, _| text[offset..text.len().min(offset + size)].as_bytes()
+}
