@@ -569,6 +569,27 @@ recur:
   return ts_node__null();
 }
 
+const char *ts_node_field_name_for_child(TSNode self, uint32_t child_index) {
+  const TSFieldMapEntry *field_map_start = NULL, *field_map_end = NULL;
+  if (!ts_node_child_count(self)) {
+    return NULL;
+  }
+
+  ts_language_field_map(
+    self.tree->language,
+    ts_node__subtree(self).ptr->production_id,
+    &field_map_start,
+    &field_map_end
+  );
+
+  for (const TSFieldMapEntry *i = field_map_start; i < field_map_end; i++) {
+    if (i->child_index == child_index) {
+      return self.tree->language->field_names[i->field_id];
+    }
+  }
+  return NULL;
+}
+
 TSNode ts_node_child_by_field_name(
   TSNode self,
   const char *name,
